% This program replicates the computations in Carroll's model using my own
% program in order to time it accordingly. It uses loops everywhere so that
% the timing goes in agreement with the model we have.

clear all;
format long;
tic
beta=0.96;   % time preference
rho=2;       % coefficient of relative risk aversion
n=100;        % number of grid points
G=1.01;      % permanent income growth rate
dep=0.90;    % depreciation rate
curlyepsilon=0.36;  % Cobb-Douglas production function parameter
permval=[0.90,1.00,1.10];   % permanent shock values
permprob=[0.25,0.50,0.25];  % permanent shock probabilities
kss=((((G^rho)/(beta*dep))-1)/curlyepsilon)^(1/(curlyepsilon-1));

%Grid on market resources
%Mgrid=exp(linspace(0.00,log(11),n))-1; % set up grid points
Mgrid=linspace(0.00,10,n);
%Initial guess for C
C1=0:n-1;
C1=0.1*C1;

conv=100;
iter=0;
while conv>1e-6
    iter=iter+1;
    for j=1:length(Mgrid)
        if j>1
        C2(j)=csolve(@(C) eulerequation(C,Mgrid(j),beta,rho,G,dep,curlyepsilon,permval,permprob,C1,Mgrid), C1(j), [], 0.000001, 500);
        else
        C2(j)=0;
        end
    end
    conv=max(abs(C2-C1));
    C1=C2;
end
toc

figure;
plot(Mgrid,C1)
xlabel('m')
ylabel('c')
title('Consumption Function for Macro Model')
axis([0 5 0 2])

%This part of the program computes the Euler Equation errors.

EUP=0;
for j=1:length(permval)
        k=(Mgrid-C1)*dep/(G*permval(j));
        R=1+curlyepsilon*k.^(curlyepsilon-1);
        W=(1-curlyepsilon)*(k.^curlyepsilon);
        mtomorrow=R.*k+W;
        mtomorrow(1)=0; %To deal with no resources no consumption.
        Cnextp=interp1(Mgrid,C1,mtomorrow,'linear','extrap');
        EUP=EUP+dep*beta*R.*(G*permval(j)*Cnextp).^-rho*permprob(j);
end

EEerror=1-(EUP.^(-1/rho))./C1;
EEerrormax=max(abs(EEerror))
EEerrorlog=log10(abs(EEerror));
EEerrormaxlog=max(EEerrorlog)